@@ -102,7 +102,7 @@ void rwMatrix(int *stop,int *row,double *w,double *X,int *n,int *p,int *trans,do
 void in_out(double *bx, double *by, double *break_code, double *x,double *y,int *in, int *nb, int *n);
 void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol,int *nt);
 void RuniqueCombs(double *X,int *ind,int *r, int *c);
-void  RPCLS(double *Xd,double *pd,double *yd, double *wd,double *Aind,double *bd,double *Afd,double *Hd,double *Sd,int *off,int *dim,double *theta, int *m,int *nar);
+void  RPCLS(double *Xd,double *pd,double *yd, double *wd,double *Aind,double *bd,double *Afd,double *Sd,int *off,int *dim,double *theta, int *m,int *nar);
 void RMonoCon(double *Ad,double *bd,double *xd,int *control,double *lower,double *upper,int *n);
 /*void MinimumSeparation(double *gx,double *gy,int *gn,double *dx,double *dy, int *dn,double *dist);*/
 void MinimumSeparation(double *x,int *n, int *d,double *t,int *m,double *dist);
@@ -190,9 +190,12 @@ typedef struct {
 
 void k_newn_work(double *Xm,kdtree_type kd,double *X,double *dist,int *ni,int*m,int *n,int *d,int *k);
 void k_nn(double *X,double *dist,double *a,int *ni,int *n,int *d,int *k,int *get_a);
-void Rkdtree(double *X,int *n, int *d,int *idat,double *ddat);
-void Rkdnearest(double *X,int *idat,double *ddat,int *n,double *x, int *m, int *ni, double *dist,int *k);
-void Rkradius(double *r,int *idat,double *ddat,double *X,double *x,int *m,int *off,int *ni,int *op);
+//void Rkdtree(double *X,int *n, int *d,int *idat,double *ddat);
+SEXP Rkdtree(SEXP x);
+//void Rkdnearest(double *X,int *idat,double *ddat,int *n,double *x, int *m, int *ni, double *dist,int *k);
+SEXP Rkdnearest(SEXP Xr, SEXP xr,SEXP k);
+//void Rkradius(double *r,int *idat,double *ddat,double *X,double *x,int *m,int *off,int *ni,int *op);
+SEXP Rkradius(SEXP Xr, SEXP xr,SEXP rr,SEXP offr);
 double xidist(double *x,double *X,int i,int d, int n);
 int closest(kdtree_type *kd, double *X,double *x,int n,int *ex,int nex);
 void kd_tree(double *X,int *n, int *d,kdtree_type *kd);
